#!/usr/bin/env python3

# Copyright (c) 2022, harens
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of seaport nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Commands related to managing the clone macports-ports repo."""

import os
import subprocess

import click
from beartype import beartype
from beartype.typing import Tuple

from seaport._clipboard.checks import user_path
from seaport._clipboard.format import format_subprocess


@beartype
def sync_fork(location: str) -> None:
    """Update the cloned repo with upstream.

    Based on https://docs.github.com/en/free-pro-team@latest/github/collaborating-with-issues-and-pull-requests/syncing-a-fork

    Args:
        location: Where the macports-ports repo is located
    """
    os.chdir(f"{location}/macports-ports")
    subprocess.run([f"{user_path()}/git", "checkout", "-f", "master"], check=True)
    subprocess.run([f"{user_path()}/git", "fetch", "upstream"], check=True)
    subprocess.run([f"{user_path()}/git", "merge", "upstream/master"], check=True)
    subprocess.run([f"{user_path()}/git", "push"], check=True)


@beartype
def pr_variables() -> Tuple[str, str]:
    """Determines macOS and Xcode version numbers for pr template.

    If Xcode isn't installed, it outputs the Xcode CLT version.

    Returns:
        Tuple[str, str]: The macOS version and Xcode version
    """
    mac_version = " ".join(
        [
            format_subprocess([f"{user_path()}/sw_vers", "-productVersion"]),
            format_subprocess([f"{user_path()}/sw_vers", "-buildVersion"]),
        ]
    )

    try:
        xcode_version = format_subprocess(
            [f"{user_path()}/xcodebuild", "-version"]
        ).replace("\nBuild version", "")
    except subprocess.CalledProcessError:
        # If Xcode isn't installed
        click.secho("⏩ Using Command Line Tools instead", fg="cyan")
        xcode_version = format_subprocess([f"{user_path()}/xcode-select", "--version"])

    return mac_version, xcode_version



#!/usr/bin/env python3

# Copyright (c) 2022, harens
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of seaport nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Functions related to determining the current and new checksums."""

import shutil
import sys
import tempfile
import urllib.request
from pathlib import Path

import click
from beartype import beartype
from beartype.typing import Tuple

from seaport._clipboard.checks import user_path
from seaport._clipboard.format import format_subprocess
from seaport._clipboard.portfile.portfile_numbers import undo_revision


@beartype
def new_checksums(website: str) -> Tuple[str, str, str]:
    """Generate checksums of file downloaded from website.

    Args:
        website: Where to download the new file from

    Examples:
        >>> from seaport._clipboard.portfile.checksums import new_checksums
        >>> new_checksums("https://files.pythonhosted.org/packages/source/r/rich/rich-9.10.0.tar.gz")
        🔻 Downloading from https://files.pythonhosted.org/packages/source/r/rich/rich-9.10.0.tar.gz
        ('e0f2db62a52536ee32f6f584a47536465872cae2b94887cf1f080fb9eaa13eb2', '3f8be5bb8220538ed2f7953a25d829584fa3b379', '172290')
        >>> try:
        ...     new_checksums("I_don't_exist_and_so_will_fail")
        ... except SystemExit:
        ...     pass
        🔻 Downloading from I_don't_exist_and_so_will_fail
        Couldn't determine the new url. Modify the url above and use the --url flag to set it manually

    Returns:
        Tuple[str, str, str]: A tuple of strings representing the new checksums
    """
    download_dir = tempfile.TemporaryDirectory()
    download_location = f"{download_dir.name}/download"

    # Download the file from `url` and save it locally under `file_name`:
    # Originally urllib.request.urlretrieve(website, download_location), but this is depreciated
    # Credit https://stackoverflow.com/a/7244263
    click.secho(f"🔻 Downloading from {website}", fg="cyan")
    try:
        with urllib.request.urlopen(website) as response, open(
            download_location, "wb"
        ) as out_file:
            shutil.copyfileobj(response, out_file)
    except (urllib.error.HTTPError, urllib.error.URLError, ValueError):
        click.secho(
            "Couldn't determine the new url. Modify the url above and use the --url flag to set it manually",
            fg="red",
        )
        sys.exit(1)

    # sha256 flag added even though it's the default
    # Otherwise sometimes doesn't return sha256
    # TODO: Repalce the subprocess with native python functions
    sha256 = format_subprocess(
        [f"{user_path()}/openssl", "dgst", "-sha256", download_location]
    ).split(" ")[-1]
    rmd160 = format_subprocess(
        [f"{user_path()}/openssl", "dgst", "-rmd160", download_location]
    ).split(" ")[-1]
    size = str(Path(download_location).stat().st_size)

    download_dir.cleanup()

    return sha256, rmd160, size


@beartype
def replace_checksums(
    file_contents: str,
    old_sums: Tuple[str, str, str, str],
    new_sums: Tuple[str, str, str, str],
) -> str:
    """Replaces the old checksums with the new ones.

    Args:
        file_contents: The old contents of the file
        old_sums: The old checksums that are in file_contents
        new_sums: The new checksums that will replace the old ones

    Examples:
        >>> from seaport._clipboard.portfile.checksums import replace_checksums
        >>> replace_checksums(
        ... "replace this oldrmd, oldsha, oldsize and oldversion with new ones please",
        ... ("oldrmd", "oldsha", "oldsize", "oldversion"),
        ... ("newrmd", "newsha", "newsize", "newversion"),
        ... )
        ⏪️ Changing revision numbers
        No changes necessary
        'replace this newrmd, newsha, newsize and newversion with new ones please'

    Returns:
        A string representing the portfile contents with the new checksums
    """
    # Bump revision numbers to 0
    new_contents: str = undo_revision(file_contents)

    # Replace first instances only
    # Iterate over Checksums and version number
    for i in range(4):
        new_contents = new_contents.replace(old_sums[i], new_sums[i], 1)

    return new_contents



#!/usr/bin/env python3

# Copyright (c) 2022, harens
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of seaport nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Functions related to bumping revision and version numbers of a portfile."""

import re
import sys
from typing import Optional

import click
from beartype import beartype

from seaport.portfile import Port


@beartype
def undo_revision(text: str) -> str:
    """Make revision numbers 0.

    Note that this supports a single revision number or multiple revision numbers that are 0.
    If there are multiple revision numbers, and one of them is not 0, it raises SystemExit 1

    Args:
        text: The text of the portfile

    Examples:
        >>> from seaport._clipboard.portfile.portfile_numbers import undo_revision
        >>> undo_revision("No revision numbers here")
        ⏪️ Changing revision numbers
        No changes necessary
        'No revision numbers here'
        >>> try:
        ...     undo_revision("revision 1  revision 1")
        ... except SystemExit:
        ...     pass
        ⏪️ Changing revision numbers
        Multiple revision numbers found. Unsure which to reduce to 0
        >>> undo_revision("revision 1")
        ⏪️ Changing revision numbers
        Revision number changed
        'revision 0'


    Returns:
        str: The text with version numbers decremented to 0

    """
    click.secho("⏪️ Changing revision numbers", fg="cyan")

    # Counts no. of revision numbers greater than 0
    # Assumes revision number doesn't exceed 9
    need_changed = len(re.findall(r"revision\s*[1-9]", text))
    total = len(re.findall(r"revision\s*", text))

    # If there are no revisions greater than 1, do nothing
    if need_changed == 0:
        click.echo("No changes necessary")
        return text
    # If there are multiple revision numbers, we don't know which one to change
    # If all of the revision numbers are 0, this is accounted for in the last if statement
    if total > 1:
        click.secho(
            "Multiple revision numbers found. Unsure which to reduce to 0", fg="red"
        )
        sys.exit(1)

    # Implies need_changed == 1

    # Takes the original revision as a string
    # We know original_revision will not be of type none (hence mypy ignore)
    # This is since need_changed is equal to 1
    original_revision = re.search(r"revision\s*[1-9]", text).group(0)  # type: ignore
    # Replaces the number with 0
    new_revision = original_revision[:-1] + "0"
    click.echo("Revision number changed")
    return text.replace(original_revision, new_revision)


# TODO: The doctest isn't reproducible, since the version number might change.
@beartype
def new_version(port: Port, stated: Optional[str], new: bool = False) -> str:
    """Determines livecheck version, and sees whether already up-to-date.

    Args:
        port: The port class
        stated: The user's new version via --bump
        new: If the port is new or not

    Examples:
        >>> from seaport.portfile import Port
        >>> from seaport._clipboard.portfile.portfile_numbers import new_version
        >>> port = Port("py-base91")
        >>> # If the port is a new one
        >>> new_version(port, "1.2.0", True)
        '1.0.1'
        >>> # If the version has been stated
        >>> new_version(port, "2.0")
        '2.0'

    Returns:
        str: Either the latest version number or the user's custom one.

    """
    # TODO: This will need some serious refactoring at some point
    # If it's a new port, essentially ignore version checks
    # Returns the current version from the portfile
    if new:
        return port.version

    # Determines new version number if none manually specified
    if stated is None:  # None used rather than "is not" to make mypy happy
        stated = port.livecheck()

    if stated == port.version:
        click.secho(
            f"{port.name} is already up-to-date ({port.version}) or there's no livecheck available",
            fg="red",
        )
        click.secho("Please manually specify the version using --bump", fg="red")
        sys.exit(1)

    # Credit to @herbygillot
    # See https://github.com/macports/macports-ports/pull/9589#issuecomment-753309298
    # alpha/beta/rc version detected on a port that isn't -devel
    devel_versions = ["alpha", "beta", "rc", "devel", "dev", "unstable"]
    if "-devel" not in port.name and any(item in stated for item in devel_versions):
        if not click.confirm(
            f"{port.name} is not a devel port, but the new version ({stated}) is a devel build. Do you wish to continue?"
        ):
            click.echo("You can specify a different version using --bump")
            sys.exit(1)

    return stated



#!/usr/bin/env python3

# Copyright (c) 2022, harens
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of seaport nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Defensive programming functions."""

from beartype import beartype

from seaport._clipboard.format import format_subprocess


# TODO: This will need major refactoring (it's a mess)
# It also kind of defeats the purpose of the bandit error it's meant to solve
# TODO: This test will fail if non-default macports path is used
@beartype
def user_path(port: bool = False, third_party: bool = False) -> str:
    """Determines the path to prevent starting a process with a partial executable path.

    Examples:
        >>> from seaport._clipboard.checks import user_path
        >>> # If it's a port command (e.g. port lint)
        >>> user_path(True)
        '/opt/local/bin'
        >>> # If it's a standard system command
        >>> user_path()
        '/usr/bin'

    Args:
        port: Whether to output the path of the port cmd
        third_party: Whether the dependency isn't installed by default

    Returns:
        A string representing the path

    Path to run system commands (e.g. git)
    See https://bandit.readthedocs.io/en/latest/plugins/b607_start_process_with_partial_path.html
    """
    # no forward slash at end for Bandit B607

    port_path = format_subprocess(["/usr/bin/which", "port"])
    port_prefix = port_path.split("bin")[0] + "bin"

    if port:
        return port_prefix

    if third_party:
        # Cannot use port_path in case not installed by MacPorts (e.g. Homebrew)
        seaport_path = format_subprocess(["/usr/bin/which", "seaport"])

        if "Python" not in seaport_path and "virtualenvs" not in seaport_path:
            # Can't run system commands from python path
            return seaport_path.split("bin")[0] + "bin"

        # Default to standard MacPorts prefix
        return port_prefix

    return "/usr/bin"



#!/usr/bin/env python3

# Copyright (c) 2022, harens
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of seaport nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Functions related to parsing the portfile."""

import os
import sys

import click
from beartype import beartype
from beartype.typing import Tuple

from seaport._clipboard.format import format_subprocess


@beartype
def new_contents() -> Tuple[str, str, str]:
    """Determines the new contents and version number of a portfile.

    Returns:
        (str, str, str): The contents of the new portfile, version number and category
    """
    # Determine the output of the clip function from clipboard
    # Newline added since clipboard removes it
    contents = format_subprocess(["pbpaste"]) + "\n"

    if "port" not in contents.lower():
        click.secho("Cannot retrieve portfile contents from clipboard", fg="red")
        sys.exit(1)

    # Get updated version number from clip
    # Separate var to get mypy to work
    # This is since os.getenv is Optional[str]
    bump = os.getenv("BUMP")
    category = os.getenv("CATEGORY")

    if bump is None or category is None:
        click.secho("Environment variables overwritten", fg="red")
        sys.exit(1)

    return contents, bump, category



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['seaport',
 'seaport._clipboard',
 'seaport._clipboard.portfile',
 'seaport._pull_request']

package_data = \
{'': ['*'], 'seaport': ['_autocomplete/*']}

install_requires = \
['beartype>=0.12.0,<0.13.0', 'click>=8.1.3,<9.0.0']

entry_points = \
{'console_scripts': ['seaport = seaport._init:seaport']}

setup_kwargs = {
    'name': 'seaport',
    'version': '0.8.0',
    'description': 'The modern MacPorts portfile updater',
    'long_description': '🌊 seaport\n==========\n\n|ci-badge| |rtd-badge| |cov-badge|\n\nThe modern `MacPorts <https://www.macports.org>`_ portfile updater.\n\n.. code-block::\n\n    > seaport clip gping\n    🌊 Starting seaport...\n    👍 New version is 1.2.0-post\n    🔻 Downloading from https://github.com/orf/gping/tarball/v1.2.0-post/gping-1.2.0-post.tar.gz\n    🔎 Checksums:\n    Old rmd160: 8b274132c8389ec560f213007368c7f521fdf682\n    New rmd160: 4a614e35d4e1e496871ee2b270ba8836f84650c6\n    Old sha256: 1879b37f811c09e43d3759ccd97d9c8b432f06c75a27025cfa09404abdeda8f5\n    New sha256: 1008306e8293e7c59125de02e2baa6a17bc1c10de1daba2247bfc789eaf34ff5\n    Old size: 853432\n    New size: 853450\n    ⏪️ Changing revision numbers\n    No changes necessary\n    📋 The contents of the portfile have been copied to your clipboard!\n\nInstallation\n------------\n\nMacPorts 🍎\n************\n\n.. code-block::\n\n    sudo port install seaport\n\nPyPi 🐍\n********\n\n.. code-block::\n\n    pip3 install seaport\n    # Optionally, install GitHub CLI to send PRs\n    sudo port install gh\n\n⚡️ Features\n--------------\n\n🖥 `Command Line Tool <https://seaport.readthedocs.io/en/stable/overview.html>`_\n*********************************************************************************\n\n* ⏩ **Automatically determines new version numbers and checksums** for MacPorts portfiles.\n* 📋 **Copies the changes to your clipboard**, and optionally **sends a PR to update them**.\n* 🔎 Contains **additional checking functionality**, such as running tests, linting and installing the updated program.\n\n🐍 `Python API <https://seaport.readthedocs.io/en/stable/reference.html>`_\n****************************************************************************\n\n* 📚 Library for convenient access to portfile information. Easily import as a Python module for your project.\n* ⌨️ `PEP 561 compatible <https://www.python.org/dev/peps/pep-0561>`_, with built in support for type checking.\n*  📦 Works out of the box with all `supported Python versions <https://endoflife.date/python>`_ (3.7-3.11).\n\nTo find out more, please read the `Documentation <https://seaport.rtfd.io/>`_.\n\n🤔 How to use seaport\n----------------------\n\nFor simple ports with straightforward updates, use :code:`seaport pr example_port`.\nThis sends a PR with the updated portfile and automatically fills in the PR template for you.\n\nFor ports that require some manual changes, use :code:`seaport clip example_port`.\nThis updates the version number and checksums so you don\'t have to. 😎\n\nBe sure to check out the `flags overview <https://seaport.readthedocs.io/en/stable/overview.html>`_ for information on additional features.\n\n🔥 seaport vs port bump\n-------------------------\n\n.. list-table::\n   :widths: 25 25 25\n   :header-rows: 1\n\n   * - Features\n     - 🌊 seaport\n     - 🛼 port bump\n   * - 🔒 Updates checksums\n     - ✅\n     - ✅\n   * - 📚 Updates the revision number\n     - ✅\n     - ✅\n   * - 📝 Can write changes to the original file\n     - ✅\n     - ✅\n   * - ⏮ Can update portfile to a specific version\n     - ✅\n     - ✅\n   * - 🔮 Updates the version number via livecheck\n     - ✅\n     - ❌\n   * - 🚀 Can send a pull request (both for updated and new ports)\n     - ✅\n     - ❌\n   * - 🧪 Can lint/test/install the port to check if the update works\n     - ✅\n     - ❌\n   * - 📋 Copies changes to clipboard\n     - ✅\n     - ❌\n   * - 🌎 Can both manually and automatically set the url to download from\n     - ✅\n     - ❌\n\n🔨 Contributing\n---------------\n\n- Issue Tracker: `<https://github.com/harens/seaport/issues>`_\n- Source Code: `<https://github.com/harens/seaport>`_\n\nAny change, big or small, that you think can help improve this project is more than welcome 🎉.\n\nAs well as this, feel free to open an issue with any new suggestions or bug reports. Every contribution is appreciated.\n\nFor more information, please read our `contributing page <https://seaport.readthedocs.io/en/latest/contributing.html>`_ on how to get started.\n\n©️ License\n----------\n\nSimilar to other MacPorts-based projects, seaport is licensed under the `BSD 3-Clause "New" or "Revised" License <https://github.com/harens/seaport/blob/master/LICENSE>`_.\n\n📒 Notice of Non-Affiliation and Disclaimer\n-------------------------------------------\n\nThis project is not affiliated, associated, authorized, endorsed by, or in any way officially connected with the MacPorts Project, or any of its subsidiaries or its affiliates. The official MacPorts Project website can be found at `<https://www.macports.org>`_.\n\nThe name MacPorts as well as related names, marks, emblems and images are registered trademarks of their respective owners.\n\n.. |ci-badge| image:: https://img.shields.io/github/actions/workflow/status/harens/seaport/test.yml?logo=github&style=flat-square\n   :target: https://github.com/harens/seaport/actions\n   :alt: GitHub Workflow Status\n.. |rtd-badge| image:: https://img.shields.io/readthedocs/seaport?logo=read%20the%20docs&style=flat-square\n   :target: https://seaport.rtfd.io/\n   :alt: Read the Docs\n.. |cov-badge| image:: https://img.shields.io/codecov/c/github/harens/seaport?logo=codecov&style=flat-square\n   :target: https://codecov.io/gh/harens/seaport\n   :alt: Codecov\n',
    'author': 'harens',
    'author_email': 'harensdeveloper@gmail.com',
    'maintainer': 'harens',
    'maintainer_email': 'harensdeveloper@gmail.com',
    'url': 'https://seaport.rtfd.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
